@@ -5,7 +5,7 @@ use URI;
 use Carp ();
 
 use vars qw($VERSION);
-$VERSION = "5.811";
+$VERSION = "5.814";
 
 my %form_tags = map {$_ => 1} qw(input textarea button select option);
 
@@ -113,10 +113,7 @@ sub parse
 
     require HTML::TokeParser;
     my $p = HTML::TokeParser->new(ref($html) ? $html->decoded_content(ref => 1) : \$html);
-    eval {
-	# optimization
-	$p->report_tags(qw(form input textarea select optgroup option keygen label button));
-    };
+    die "Failed to create HTML::TokeParser object" unless $p;
 
     my $base_uri = delete $opt{base};
     my $verbose = delete $opt{verbose};
@@ -149,6 +146,7 @@ sub parse
 			     $action,
 			     $attr->{'enctype'});
 	    $f->{attr} = $attr;
+            %openselect = ();
 	    push(@forms, $f);
 	    my(%labels, $current_label);
 	    while (my $t = $p->get_tag) {
@@ -1188,13 +1186,13 @@ sub check
 sub possible_values
 {
     my $self = shift;
-    map $_->{value}, @{$self->{menu}};
+    map $_->{value}, grep !$_->{disabled}, @{$self->{menu}};
 }
 
 sub other_possible_values
 {
     my $self = shift;
-    map $_->{value}, grep !$_->{seen}, @{$self->{menu}};
+    map $_->{value}, grep !$_->{seen} && !$_->{disabled}, @{$self->{menu}};
 }
 
 sub value_names {
