@@ -1,12 +1,10 @@
 package HTTP::Headers;
 
-# $Id: Headers.pm,v 1.50 2004/04/06 22:30:52 gisle Exp $
-
 use strict;
 use Carp ();
 
 use vars qw($VERSION $TRANSLATE_UNDERSCORE);
-$VERSION = sprintf("%d.%02d", q$Revision: 1.50 $ =~ /(\d+)\.(\d+)/);
+$VERSION = "5.815";
 
 # The $TRANSLATE_UNDERSCORE variable controls whether '_' can be used
 # as a replacement for '-' in header field names.
@@ -18,22 +16,37 @@ $TRANSLATE_UNDERSCORE = 1 unless defined $TRANSLATE_UNDERSCORE;
 #    - Response-Headers
 #    - Entity-Headers
 
-my @header_order = qw(
+my @general_headers = qw(
    Cache-Control Connection Date Pragma Trailer Transfer-Encoding Upgrade
    Via Warning
+);
 
+my @request_headers = qw(
    Accept Accept-Charset Accept-Encoding Accept-Language
    Authorization Expect From Host
    If-Match If-Modified-Since If-None-Match If-Range If-Unmodified-Since
    Max-Forwards Proxy-Authorization Range Referer TE User-Agent
+);
 
+my @response_headers = qw(
    Accept-Ranges Age ETag Location Proxy-Authenticate Retry-After Server
    Vary WWW-Authenticate
+);
 
+my @entity_headers = qw(
    Allow Content-Encoding Content-Language Content-Length Content-Location
    Content-MD5 Content-Range Content-Type Expires Last-Modified
 );
 
+my %entity_header = map { lc($_) => 1 } @entity_headers;
+
+my @header_order = (
+   @general_headers,
+   @request_headers,
+   @response_headers,
+   @entity_headers,
+);
+
 # Make alternative representations of @header_order.  This is used
 # for sorting and case matching.
 my %header_order;
@@ -54,7 +67,7 @@ sub new
 {
     my($class) = shift;
     my $self = bless {}, $class;
-    $self->header(@_); # set up initial headers
+    $self->header(@_) if @_; # set up initial headers
     $self;
 }
 
@@ -62,9 +75,13 @@ sub new
 sub header
 {
     my $self = shift;
+    Carp::croak('Usage: $h->header($field, ...)') unless @_;
     my(@old);
-    while (my($field, $val) = splice(@_, 0, 2)) {
-	@old = $self->_header($field, $val);
+    my %seen;
+    while (@_) {
+	my $field = shift;
+        my $op = @_ ? ($seen{lc($field)}++ ? 'PUSH' : 'SET') : 'GET';
+	@old = $self->_header($field, shift, $op);
     }
     return @old if wantarray;
     return $old[0] if @old <= 1;
@@ -98,7 +115,7 @@ sub remove_header
     my $field;
     my @values;
     foreach $field (@fields) {
-	$field =~ tr/_/-/ if $TRANSLATE_UNDERSCORE;
+	$field =~ tr/_/-/ if $field !~ /^:/ && $TRANSLATE_UNDERSCORE;
 	my $v = delete $self->{lc $field};
 	push(@values, ref($v) eq 'ARRAY' ? @$v : $v) if defined $v;
     }
@@ -109,13 +126,13 @@ sub remove_content_headers
 {
     my $self = shift;
     unless (defined(wantarray)) {
-	# simplify
-	delete @$self{grep /^content-/, keys %$self};
+	# fast branch that does not create return object
+	delete @$self{grep $entity_header{$_} || /^content-/, keys %$self};
 	return;
     }
 
     my $c = ref($self)->new;
-    for my $f (grep /^content-/, keys %$self) {
+    for my $f (grep $entity_header{$_} || /^content-/, keys %$self) {
 	$c->{$f} = delete $self->{$f};
     }
     $c;
@@ -125,42 +142,59 @@ sub remove_content_headers
 sub _header
 {
     my($self, $field, $val, $op) = @_;
-    $field =~ tr/_/-/ if $TRANSLATE_UNDERSCORE;
 
     # $push is only used interally sub push_header
     Carp::croak('Need a field name') unless length($field);
 
-    my $lc_field = lc $field;
-    unless(defined $standard_case{$lc_field}) {
-	# generate a %standard_case entry for this field
-	$field =~ s/\b(\w)/\u$1/g;
-	$standard_case{$lc_field} = $field;
+    unless ($field =~ /^:/) {
+	$field =~ tr/_/-/ if $TRANSLATE_UNDERSCORE;
+	my $old = $field;
+	$field = lc $field;
+	unless(defined $standard_case{$field}) {
+	    # generate a %standard_case entry for this field
+	    $old =~ s/\b(\w)/\u$1/g;
+	    $standard_case{$field} = $old;
+	}
     }
 
-    my $h = $self->{$lc_field};
+    my $h = $self->{$field};
     my @old = ref($h) eq 'ARRAY' ? @$h : (defined($h) ? ($h) : ());
 
-    $op ||= "";
-    $val = undef if $op eq 'INIT' && @old;
-    if (defined($val)) {
-	my @new = ($op eq 'PUSH') ? @old : ();
-	if (ref($val) ne 'ARRAY') {
-	    push(@new, $val);
+    $op ||= defined($val) ? 'SET' : 'GET';
+    unless ($op eq 'GET' || ($op eq 'INIT' && @old)) {
+	if (defined($val)) {
+	    my @new = ($op eq 'PUSH') ? @old : ();
+	    if (ref($val) ne 'ARRAY') {
+		push(@new, $val);
+	    }
+	    else {
+		push(@new, @$val);
+	    }
+	    $self->{$field} = @new > 1 ? \@new : $new[0];
 	}
-	else {
-	    push(@new, @$val);
+	elsif ($op ne 'PUSH') {
+	    delete $self->{$field};
 	}
-	$self->{$lc_field} = @new > 1 ? \@new : $new[0];
     }
     @old;
 }
 
 
-# Compare function which makes it easy to sort headers in the
-# recommended "Good Practice" order.
-sub _header_cmp
+sub _sorted_field_names
 {
-    ($header_order{$a} || 999) <=> ($header_order{$b} || 999) || $a cmp $b;
+    my $self = shift;
+    return sort {
+        ($header_order{$a} || 999) <=> ($header_order{$b} || 999) ||
+         $a cmp $b
+    } keys %$self
+}
+
+
+sub header_field_names {
+    my $self = shift;
+    return map $standard_case{$_} || $_, $self->_sorted_field_names
+	if wantarray;
+    return keys %$self;
 }
 
 
@@ -168,7 +202,7 @@ sub scan
 {
     my($self, $sub) = @_;
     my $key;
-    foreach $key (sort _header_cmp keys %$self) {
+    foreach $key ($self->_sorted_field_names) {
         next if $key =~ /^_/;
 	my $vals = $self->{$key};
 	if (ref($vals) eq 'ARRAY') {
@@ -192,6 +226,7 @@ sub as_string
     my @result = ();
     $self->scan(sub {
 	my($field, $val) = @_;
+	$field =~ s/^://;
 	if ($val =~ /\n/) {
 	    # must handle header values with embedded newlines with care
 	    $val =~ s/\s+$//;          # trailing newlines and space must go
@@ -223,6 +258,7 @@ sub _date_header
     if (defined $time) {
 	$self->_header($header, HTTP::Date::time2str($time));
     }
+    $old =~ s/;.*// if defined($old);
     HTTP::Date::str2time($old);
 }
 
@@ -233,7 +269,7 @@ sub if_modified_since   { shift->_date_header('If-Modified-Since',   @_); }
 sub if_unmodified_since { shift->_date_header('If-Unmodified-Since', @_); }
 sub last_modified       { shift->_date_header('Last-Modified',       @_); }
 
-# This is used as a private LWP extention.  The Client-Date header is
+# This is used as a private LWP extension.  The Client-Date header is
 # added as a timestamp to a response when it has been received.
 sub client_date         { shift->_date_header('Client-Date',         @_); }
 
@@ -247,10 +283,45 @@ sub client_date         { shift->_date_header('Client-Date',         @_); }
 sub content_type      {
   my $ct = (shift->_header('Content-Type', @_))[0];
   return '' unless defined($ct) && length($ct);
-  my @ct = split(/\s*;\s*/, lc($ct));
+  my @ct = split(/;\s*/, $ct, 2);
+  for ($ct[0]) {
+      s/\s+//g;
+      $_ = lc($_);
+  }
   wantarray ? @ct : $ct[0];
 }
 
+sub content_is_html {
+    my $self = shift;
+    return $self->content_type eq 'text/html' || $self->content_is_xhtml;
+}
+
+sub content_is_xhtml {
+    my $ct = shift->content_type;
+    for (qw(application/xhtml+xml application/vnd.wap.xhtml+xml)) {
+        return 1 if $_ eq $ct;
+    }
+    return 0;
+}
+
+sub referer           {
+    my $self = shift;
+    if (@_ && $_[0] =~ /#/) {
+	# Strip fragment per RFC 2616, section 14.36.
+	my $uri = shift;
+	if (ref($uri)) {
+	    $uri = $uri->clone;
+	    $uri->fragment(undef);
+	}
+	else {
+	    $uri =~ s/\#.*//;
+	}
+	unshift @_, $uri;
+    }
+    ($self->_header('Referer', @_))[0];
+}
+*referrer = \&referer;  # on tchrist's request
+
 sub title             { (shift->_header('Title',            @_))[0] }
 sub content_encoding  { (shift->_header('Content-Encoding', @_))[0] }
 sub content_language  { (shift->_header('Content-Language', @_))[0] }
@@ -260,8 +331,6 @@ sub user_agent        { (shift->_header('User-Agent',       @_))[0] }
 sub server            { (shift->_header('Server',           @_))[0] }
 
 sub from              { (shift->_header('From',             @_))[0] }
-sub referer           { (shift->_header('Referer',          @_))[0] }
-*referrer = \&referer;  # on tchrist's request
 sub warning           { (shift->_header('Warning',          @_))[0] }
 
 sub www_authenticate  { (shift->_header('WWW-Authenticate', @_))[0] }
@@ -314,7 +383,8 @@ HTTP::Headers - Class encapsulating HTTP Message headers
 
 The C<HTTP::Headers> class encapsulates HTTP-style message headers.
 The headers consist of attribute-value pairs also called fields, which
-may be repeated, and which are printed in a particular order.
+may be repeated, and which are printed in a particular order.  The
+field names are cases insensitive.
 
 Instances of this class are usually created as member variables of the
 C<HTTP::Request> and C<HTTP::Response> classes, internal to the
@@ -337,28 +407,31 @@ attribute-value pairs as parameters to the constructor.  I<E.g.>:
 The constructor arguments are passed to the C<header> method which is
 described below.
 
+=item $h->clone
+
+Returns a copy of this C<HTTP::Headers> object.
+
 =item $h->header( $field )
 
 =item $h->header( $field => $value, ... )
 
-Get or set the value of one or more header fields.  The header field name
-($field) is not case sensitive.  To make the life easier for perl
+Get or set the value of one or more header fields.  The header field
+name ($field) is not case sensitive.  To make the life easier for perl
 users who wants to avoid quoting before the => operator, you can use
-'_' as a replacement for '-' in header names (this behaviour can be
-suppressed by setting the $HTTP::Headers::TRANSLATE_UNDERSCORE
-variable to a FALSE value).
+'_' as a replacement for '-' in header names.
 
 The header() method accepts multiple ($field => $value) pairs, which
 means that you can update several fields with a single invocation.
 
 The $value argument may be a plain string or a reference to an array
-of strings for a multi-valued field. If the $value is undefined or not
-given, then that header field will remain unchanged.
+of strings for a multi-valued field. If the $value is provided as
+C<undef> then the field is removed.  If the $value is not given, then
+that header field will remain unchanged.
 
 The old value (or values) of the last of the header fields is returned.
 If no such field exists C<undef> will be returned.
 
-A multi-valued field will be retuned as separate values in list
+A multi-valued field will be returned as separate values in list
 context and will be concatenated with ", " as separator in scalar
 context.  The HTTP spec (RFC 2616) promise that joining multiple
 values in this way will not change the semantic of a header field, but
@@ -402,7 +475,7 @@ scalars.
 
 =item $h->remove_header( $field, ... )
 
-This function removes the headers fields with the specified names.
+This function removes the header fields with the specified names.
 
 The header field names ($field) are not case sensitive and '_'
 can be used as a replacement for '-'.
@@ -415,8 +488,11 @@ possible to tell which of the returned values belonged to which field.
 
 =item $h->remove_content_headers
 
-This will remove all the headers used to describe the content of a
-message.  These header field names are prefixed with C<Content->.
+This will remove all the header fields used to describe the content of
+a message.  All header field names prefixed with C<Content-> falls
+into this category, as well as C<Allow>, C<Expires> and
+C<Last-Modified>.  RFC 2616 denote these fields as I<Entity Header
+Fields>.
 
 The return value is a new C<HTTP::Headers> object that contains the
 removed headers only.
@@ -425,6 +501,14 @@ removed headers only.
 
 This will remove all header fields.
 
+=item $h->header_field_names
+
+Returns the list of distinct names for the fields present in the
+header.  The field names have case as suggested by HTTP spec, and the
+names are returned in the recommended "Good Practice" order.
+
+In scalar context return the number of distinct field names.
+
 =item $h->scan( \&process_header_field )
 
 Apply a subroutine to each header field in turn.  The callback routine
@@ -435,35 +519,32 @@ callback routine has case as suggested by HTTP spec, and the headers
 will be visited in the recommended "Good Practice" order.
 
 Any return values of the callback routine are ignored.  The loop can
-be broken by raising an exception (C<die>).
+be broken by raising an exception (C<die>), but the caller of scan()
+would have to trap the exception itself.
 
 =item $h->as_string
 
-=item $h->as_string( $endl )
+=item $h->as_string( $eol )
 
 Return the header fields as a formatted MIME header.  Since it
 internally uses the C<scan> method to build the string, the result
 will use case as suggested by HTTP spec, and it will follow
-recommended "Good Practice" of ordering the header fieds.  Long header
+recommended "Good Practice" of ordering the header fields.  Long header
 values are not folded.
 
-The optional $endl parameter specifies the line ending sequence to
+The optional $eol parameter specifies the line ending sequence to
 use.  The default is "\n".  Embedded "\n" characters in header field
-values will be substitued with this line ending sequence.
-
-=item $h->clone
-
-Returns a copy of this C<HTTP::Headers> object.
+values will be substituted with this line ending sequence.
 
 =back
 
 =head1 CONVENIENCE METHODS
 
 The most frequently used headers can also be accessed through the
-following convenience methods.  These methods can both be used to read
+following convenience Methods.  Most of these methods can both be used to read
 and to set the value of a header.  The header value is set if you pass
 an argument to the method.  The old header value is always returned.
-If the given header did not exists then C<undef> is returned.
+If the given header did not exist then C<undef> is returned.
 
 Methods that deal with dates/times always convert their value to system
 time (seconds since Jan 1, 1970) and they also expect this kind of
@@ -513,7 +594,8 @@ content. I<E.g.>:
 
 The value returned will be converted to lower case, and potential
 parameters will be chopped off and returned as a separate value if in
-an array context.  This makes it safe to do the following:
+an array context.  If there is no such header field, then the empty
+string is returned.  This makes it safe to do the following:
 
   if ($h->content_type eq 'text/html') {
      # we enter this place even if the real header value happens to
@@ -521,6 +603,17 @@ an array context.  This makes it safe to do the following:
      ...
   }
 
+=item $h->content_is_html
+
+Returns TRUE if the Content-Type header field indicate that the
+content is some kind of HTML (including XHTML).  This method can't be
+used to set Content-Type.
+
+=item $h->content_is_xhtml
+
+Returns TRUE if the Content-Type header field indicate that the
+content is XHTML.  This method can't be used to set Content-Type.
+
 =item $h->content_encoding
 
 The Content-Encoding header field is used as a modifier to the
@@ -570,7 +663,7 @@ I<This header is no longer part of the HTTP standard.>
 =item $h->referer
 
 Used to specify the address (URI) of the document from which the
-requested resouce address was obtained.
+requested resource address was obtained.
 
 The "Free On-line Dictionary of Computing" as this to say about the
 word I<referer>:
@@ -588,6 +681,10 @@ By popular demand C<referrer> exists as an alias for this method so you
 can avoid this misspelling in your programs and still send the right
 thing on the wire.
 
+When setting the referrer, this method removes the fragment from the
+given URI if it is present, as mandated by RFC2616.  Note that
+the removal does I<not> happen automatically if using the header(),
+push_header() or init_header() methods to set the referrer.
 
 =item $h->www_authenticate
 
@@ -627,9 +724,24 @@ header instead.
 
 =back
 
+=head1 NON-CANONICALIZED FIELD NAMES
+
+The header field name spelling is normally canonicalized including the
+'_' to '-' translation.  There are some application where this is not
+appropriate.  Prefixing field names with ':' allow you to force a
+specific spelling.  For example if you really want a header field name
+to show up as C<foo_bar> instead of "Foo-Bar", you might set it like
+this:
+
+  $h->header(":foo_bar" => 1);
+
+These field names are returned with the ':' intact for
+$h->header_field_names and the $h->scan callback, but the colons do
+not show in $h->as_string.
+
 =head1 COPYRIGHT
 
-Copyright 1995-2004 Gisle Aas.
+Copyright 1995-2005 Gisle Aas.
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
